/*
HDD Explorer AppList Editor
Copyright (c) 2006 Bruno Freitas
bootsector@ig.com.br
 
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 */

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class AppListEditor extends JFrame implements ActionListener {
    
    private JPanel pnlAplicacoes, pnlBtnAplicacoes, pnlGeracao, pnlBtnGeracao;
    private JButton btnAppIncluir, btnAppEditar, btnAppExcluir, btnSalvar, btnAbrir, btnSair, btnSobre, btnCima, btnBaixo, btnOrdena;
    private JList lstAplicacoes;
    private DefaultListModel lmAplicacoes;
    
    public AppListEditor() {
        super("HDD Explorer AppList Editor v1.1a - Java Edition");
        
        Container container = getContentPane();
        container.setLayout(new BorderLayout());
        
        // Painel de Aplicaes e seu contedo
        
        // Aplicaes
        pnlAplicacoes = new JPanel(new BorderLayout());
        pnlAplicacoes.setBorder(new TitledBorder("Aplicaes:"));
        
        lmAplicacoes = new DefaultListModel();
        lstAplicacoes = new JList(lmAplicacoes);
        lstAplicacoes.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstAplicacoes.addMouseListener(new MouseListener());
        
        // Painel de botes de Aplicaes
        pnlBtnAplicacoes = new JPanel(new GridLayout(2, 3));
        
        btnAppIncluir = new JButton("Incluir");
        btnAppIncluir.setMnemonic('I');
        btnAppIncluir.setIcon(new ImageIcon(getClass().getResource("res/img/novo.png")));
        btnAppIncluir.addActionListener(this);
        pnlBtnAplicacoes.add(btnAppIncluir);
        
        btnAppEditar = new JButton("Editar");
        btnAppEditar.setMnemonic('E');
        btnAppEditar.setIcon(new ImageIcon(getClass().getResource("res/img/editar.png")));
        btnAppEditar.addActionListener(this);
        pnlBtnAplicacoes.add(btnAppEditar);
        
        btnAppExcluir = new JButton("Excluir");
        btnAppExcluir.setMnemonic('x');
        btnAppExcluir.setIcon(new ImageIcon(getClass().getResource("res/img/deletar.png")));
        btnAppExcluir.addActionListener(this);
        pnlBtnAplicacoes.add(btnAppExcluir);
        
        btnCima = new JButton("Para cima");
        btnCima.setMnemonic('c');
        btnCima.setIcon(new ImageIcon(getClass().getResource("res/img/arrowup.png")));
        btnCima.addActionListener(this);
        pnlBtnAplicacoes.add(btnCima);
        
        btnBaixo = new JButton("Para baixo");
        btnBaixo.setMnemonic('b');
        btnBaixo.setIcon(new ImageIcon(getClass().getResource("res/img/arrowdown.png")));
        btnBaixo.addActionListener(this);
        pnlBtnAplicacoes.add(btnBaixo);
        
        btnOrdena = new JButton("Ordernar");
        btnOrdena.setMnemonic('r');
        btnOrdena.setIcon(new ImageIcon(getClass().getResource("res/img/az.png")));
        btnOrdena.addActionListener(this);
        pnlBtnAplicacoes.add(btnOrdena);
        
        pnlAplicacoes.add(new JScrollPane(lstAplicacoes), BorderLayout.CENTER);
        pnlAplicacoes.add(pnlBtnAplicacoes, BorderLayout.SOUTH);
        
        // Gerao
        pnlGeracao = new JPanel(new BorderLayout());
        pnlGeracao.setBorder(new TitledBorder("Opes disponveis:"));
        
        pnlBtnGeracao = new JPanel(new GridLayout(4, 1));
        
        btnAbrir = new JButton("Abrir APPLIST.DAT");
        btnAbrir.setMnemonic('A');
        btnAbrir.setIcon(new ImageIcon(getClass().getResource("res/img/pasta.png")));
        btnAbrir.addActionListener(this);
        pnlBtnGeracao.add(btnAbrir);
        
        btnSalvar = new JButton("Gravar APPLIST.DAT");
        btnSalvar.setMnemonic('G');
        btnSalvar.setIcon(new ImageIcon(getClass().getResource("res/img/gerar.png")));
        btnSalvar.addActionListener(this);
        pnlBtnGeracao.add(btnSalvar);
        
        btnSobre = new JButton("Sobre este programa...");
        btnSobre.setMnemonic('o');
        btnSobre.setIcon(new ImageIcon(getClass().getResource("res/img/sobre.png")));
        btnSobre.addActionListener(this);
        pnlBtnGeracao.add(btnSobre);
        
        btnSair = new JButton("Sair");
        btnSair.setMnemonic('S');
        btnSair.setIcon(new ImageIcon(getClass().getResource("res/img/cancelar.png")));
        btnSair.addActionListener(this);
        pnlBtnGeracao.add(btnSair);
        
        pnlGeracao.add(pnlBtnGeracao, BorderLayout.SOUTH);
        
        // Adiciona os painis Aplicaes e Gerao na janela principal
        container.add(pnlAplicacoes, BorderLayout.CENTER);
        container.add(pnlGeracao, BorderLayout.SOUTH);
        
        setSize(360, 410);
        setIconImage(new ImageIcon(getClass().getResource("res/img/ps2.png")).getImage());
        setResizable(false);
        centerFrameOnScreen(this);
        setVisible(true);
    }
    
    public static void main(String args[]) {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch(Exception e) {
            throw new RuntimeException(e);
        }
        
        Font defFont = new DefaultFont().getFont(Font.PLAIN, 11);
        
        UIManager.put("Label.font", defFont);
        UIManager.put("Button.font", defFont);
        UIManager.put("List.font", defFont);
        UIManager.put("TextField.font", defFont);
        UIManager.put("TitledBorder.font", defFont);
        
        AppListEditor app = new AppListEditor();
        
        app.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
    
    private void centerFrameOnScreen(JFrame frame) {
        Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
        Dimension f=frame.getSize();
        
        frame.setBounds(Math.max(0, (d.width - f.width) / 2),
                Math.max(0, (d.height - f.height) / 2),
                f.width, f.height);
    }
    
    private void AppEditar() {
        if (lmAplicacoes.size() == 0) {
            JOptionPane.showMessageDialog(this, "No existem aplicaes para serem alteradas!", "Ateno", JOptionPane.WARNING_MESSAGE);
            return;
        }
        
        Aplicacao ap = (Aplicacao) lstAplicacoes.getSelectedValue();
        
        if (ap == null) {
            JOptionPane.showMessageDialog(this, "Selecione uma aplicao da lista primeiro!", "Ateno", JOptionPane.WARNING_MESSAGE);
            return;
        }
        
        FrmManAplicacao f = new FrmManAplicacao(this, "Alterao de Aplicao", ap, false);
        f.setVisible(true);
        f.dispose();
        
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == btnAppIncluir) {
            Aplicacao ap = new Aplicacao();
            FrmManAplicacao f = new FrmManAplicacao(this, "Incluso de Aplicao", ap, true);
            f.setVisible(true);
            f.dispose();
            
            if (ap.getTitulo().compareTo("") != 0) {
                lmAplicacoes.addElement(ap);
                lstAplicacoes.setSelectedValue(ap, true);
            }
        }
        
        if (e.getSource() == btnAppEditar) {
            AppEditar();
        }
        
        if (e.getSource() == btnAppExcluir) {
            if (lmAplicacoes.size() == 0) {
                JOptionPane.showMessageDialog(this, "No existem aplicaes para serem excludas!", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            Aplicacao ap = (Aplicacao) lstAplicacoes.getSelectedValue();
            
            if (ap == null) {
                JOptionPane.showMessageDialog(this, "Selecione uma aplicao da lista primeiro!", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            Object[] opcoes = {"Sim", "No"};
            if (JOptionPane.showOptionDialog(this, "Confirma excluso da aplicao selecionada?", "Confirmao", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, opcoes, opcoes[1]) == JOptionPane.YES_OPTION)
                lmAplicacoes.removeElement(ap);
        }
        
        if (e.getSource() == btnSair)
            System.exit(0);
        
        if (e.getSource() == btnSobre)
            JOptionPane.showMessageDialog(this, "Programa desenvolvido por Bruno Freitas - bootsector@ig.com.br\n\n*** Este software  FREEWARE e Open Source - http://www.brunofreitas.com/ ***\n\nAgradecimento: Damir Kolaric (skorp_jon) pelas informaes valiosas sobre o APPLIST.DAT!" , "Sobre o programa...", JOptionPane.INFORMATION_MESSAGE);
        
        if (e.getSource() == btnSalvar) {
            if (lmAplicacoes.size() == 0) {
                JOptionPane.showMessageDialog(this, "Voc deve ter pelo menos uma aplicao na lista!", "Ateno", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            try {
                JFileChooser fc = new JFileChooser(new File(new File(".").getCanonicalPath()));
                fc.setFileFilter(new AppListFileFilter());
                fc.setDialogTitle("Salvar APPLIST.DAT em...");
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                
                if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
                    AppList.GeraAppList(lmAplicacoes, fc.getSelectedFile().getAbsolutePath());
                    JOptionPane.showMessageDialog(this, "APPLIST.DAT gerado com sucesso!", "Informao", JOptionPane.INFORMATION_MESSAGE);
                }
            } catch (Exception ex) {}
        }
        
        if (e.getSource() == btnAbrir) {
            try {
                JFileChooser fc = new JFileChooser(new File(new File(".").getCanonicalPath()));
                fc.setFileFilter(new AppListFileFilter());
                fc.setDialogTitle("Abrir APPLIST.DAT...");
                
                if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                    AppList.AbreAppList(lmAplicacoes, fc.getSelectedFile().getAbsolutePath(), this);
                }
            } catch (Exception ex) {}
        }
        
        if (e.getSource() == btnCima)
            moveItemCima();
        
        if (e.getSource() == btnBaixo)
            moveItemBaixo();
        
        if (e.getSource() == btnOrdena)
            ordenaLista();
    }
    
    private void moveItemCima() {
        int indice = lstAplicacoes.getSelectedIndex();
        if (indice == -1 || indice == 0) return;
        Aplicacao item1 = (Aplicacao) lmAplicacoes.getElementAt(indice-1);
        Aplicacao item2 = (Aplicacao) lmAplicacoes.getElementAt(indice);
        lmAplicacoes.setElementAt(item2, indice-1);
        lmAplicacoes.setElementAt(item1, indice);
        lstAplicacoes.setSelectedIndex(indice-1);
        lstAplicacoes.ensureIndexIsVisible(indice-1);
    }
    
    private void moveItemBaixo() {
        int indice = lstAplicacoes.getSelectedIndex();
        if (indice == -1 || indice == lmAplicacoes.size()-1)return;
        Aplicacao item1 = (Aplicacao) lmAplicacoes.getElementAt(indice);
        Aplicacao item2 = (Aplicacao) lmAplicacoes.getElementAt(indice+1);
        lmAplicacoes.setElementAt(item2, indice);
        lmAplicacoes.setElementAt(item1, indice+1);
        lstAplicacoes.setSelectedIndex(indice+1);
        lstAplicacoes.ensureIndexIsVisible(indice+1);
    }
    
    private void ordenaLista() {
        Comparator<Aplicacao> comp = new Comparator<Aplicacao>() {
            public int compare(Aplicacao a, Aplicacao b) {
                return (a.toString().compareToIgnoreCase(b.toString()));
            }
        };
        
        Vector<Aplicacao> lista = new Vector<Aplicacao>();
        
        for (int i = 0; i < lmAplicacoes.size(); i++)
            lista.add((Aplicacao)lmAplicacoes.get(i));
        
        Collections.sort(lista, comp);
        
        lmAplicacoes.removeAllElements();
        
        for (int i = 0; i < lista.size(); i++)
            lmAplicacoes.addElement(lista.get(i));
    }
    
    private class MouseListener extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
                AppEditar();
            }
        }
    }
}
